const express = require('express');
const sqlite3 = require('sqlite3').verbose();
const bodyParser = require('body-parser');
const WebSocket = require('ws');
const { spawn } = require('child_process');
const path = require('path');

// Initialize app and database
const app = express();
const db = new sqlite3.Database('./visitors.db');

// Middleware to parse JSON requests
app.use(bodyParser.json());

// Serve static files (HTML, CSS, JS)
app.use(express.static(path.join(__dirname)));

// Endpoint to save geolocation and IP data
app.post('/save-visitor', (req, res) => {
    const { ip_address, latitude, longitude } = req.body;
    console.log('Received data:', { ip_address, latitude, longitude });

    // Insert data into SQLite
    db.run(
        `INSERT INTO visitors (ip_address, latitude, longitude) VALUES (?, ?, ?)`,
        [ip_address, latitude, longitude],
        (err) => {
            if (err) {
                console.error("Error saving visitor data:", err.message);
                res.status(500).json({ message: "Failed to save visitor data." });
            } else {
                console.log("Visitor data saved successfully.");
                res.status(200).json({ message: "Visitor data saved successfully." });
            }
        }
    );
});

// WebSocket server for CLI integration
const wss = new WebSocket.Server({ noServer: true });

wss.on('connection', (ws) => {
    console.log('Client connected');

    let cliProcess = null;

    ws.on('message', (message) => {
        const input = message.toString().trim(); // Convert Buffer to string and trim whitespace
        console.log('Browser Input:', input);

        if (input === 'start') {
            // Start the CLI game
            console.log('Starting the CLI game...');
            cliProcess = spawn('node', ['cli_rock_paper_scissors.js']);

            // Log process startup
            console.log('CLI game process spawned.');

            // Relay CLI output to the browser
            cliProcess.stdout.on('data', (data) => {
                console.log('CLI Output:', data.toString());
                ws.send(data.toString());
            });

            // Relay CLI errors to the browser
            cliProcess.stderr.on('data', (data) => {
                console.error('CLI Error:', data.toString());
                ws.send(`Error: ${data.toString()}`);
            });

            // Handle CLI process closure
            cliProcess.on('close', (code) => {
                console.log(`CLI game process exited with code ${code}`);
                ws.send('Game process has ended.');
                cliProcess = null; // Reset the CLI process reference
            });
        } else if (cliProcess) {
            // Forward browser input to the CLI process
            console.log(`Forwarding input to CLI: ${input}`);
            try {
                cliProcess.stdin.write(input + '\n'); // Write input to the CLI script
            } catch (error) {
                console.error('Failed to write input to CLI process:', error.message);
                ws.send('Error: Failed to process input.');
            }
        } else {
            console.warn('No active CLI process to forward input to.');
        }
    });

    ws.on('close', () => {
        console.log('Client disconnected');
        if (cliProcess) {
            console.log('Terminating CLI process...');
            cliProcess.kill(); // Terminate the CLI process
        }
    });
});

// Integrate WebSocket with HTTP server
const server = app.listen(3000, () => {
    console.log(`Server running on http://localhost:3000`);
});

server.on('upgrade', (request, socket, head) => {
    wss.handleUpgrade(request, socket, head, (ws) => {
        wss.emit('connection', ws, request);
    });
});
